﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using EuroManager.Presentation.WebPortal.GameViewer.Models;

namespace EuroManager.Presentation.WebPortal.GameViewer.Updates
{
    public class BallLocationUpdate : IGameUpdate
    {
        private Ball ball;
        private float frameTime;
        private float x1;
        private float y1;
        private float x2;
        private float y2;
        private float previousTime = -1;

        public BallLocationUpdate(Ball ball, float frameTime, float x1, float y1, float x2, float y2)
        {
            this.ball = ball;
            this.frameTime = frameTime;
            this.x1 = x1;
            this.y1 = y1;
            this.x2 = x2;
            this.y2 = y2;
        }

        public void Perform(float elapsedTime)
        {
            if (elapsedTime != previousTime)
            {
                previousTime = elapsedTime;

                int ballFrameIndex = (int)(elapsedTime / frameTime);
                float weight = (elapsedTime - ballFrameIndex * frameTime) / frameTime;

                ball.X = InterpolateLocation(x1, x2, weight);
                ball.Y = InterpolateLocation(y1, y2, weight);
            }
        }

        public void Revert(float elapsedTime)
        {
            Perform(elapsedTime);
        }

        private float InterpolateLocation(float location1, float location2, float weight)
        {
            return (1 - weight) * location1 + weight * location2;
        }
    }
}
